@@ -50,6 +50,7 @@ static GPid      tap_backend_run                        (const gchar *action,
                                                          GList       *content_types,
                                                          GtkWidget   *window,
                                                          GError     **error);
+static void      tap_backend_setup_display_cb           (gpointer data);
 
 
 
@@ -59,12 +60,11 @@ tap_backend_mime_ask (GList     *mime_applications,
 {
   GAppInfo                 *mime_application = NULL;
   GIcon                    *icon;
-  GtkTooltips              *tooltips;
   GtkWidget                *button;
   GtkWidget                *dialog;
   GtkWidget                *image;
   GtkWidget                *label;
-  GtkWidget                *table;
+  GtkWidget                *grid;
   GtkWidget                *bbox;
   GtkWidget                *hbox;
   GSList                   *buttons = NULL;
@@ -73,44 +73,41 @@ tap_backend_mime_ask (GList     *mime_applications,
   gchar                    *space;
   GList                    *mp;
 
-  /* allocate tooltips */
-  tooltips = gtk_tooltips_new ();
-  g_object_ref_sink (tooltips);
-
   /* prepare the dialog to query the preferred archiver for the user */
   dialog = gtk_dialog_new_with_buttons (_("Select an archive manager"),
                                         GTK_WINDOW (parent),
-                                        GTK_DIALOG_DESTROY_WITH_PARENT
-                                        | GTK_DIALOG_NO_SEPARATOR
-                                        | GTK_DIALOG_MODAL,
-                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                        GTK_STOCK_OK, GTK_RESPONSE_OK,
+                                        GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
+                                        _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                        _("_OK"), GTK_RESPONSE_OK,
                                         NULL);
   gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 
-  /* add the main table */
-  table = gtk_table_new (2, 2, FALSE);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 12);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
-  gtk_container_set_border_width (GTK_CONTAINER (table), 6);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), table, TRUE, TRUE, 0);
-  gtk_widget_show (table);
+  /* add the main grid */
+  grid = gtk_grid_new ();
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 12);
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
+  gtk_container_set_border_width (GTK_CONTAINER (grid), 6);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), grid, TRUE, TRUE, 0);
+  gtk_widget_show (grid);
 
   /* add the header image */
   image = gtk_image_new_from_icon_name ("gnome-package", GTK_ICON_SIZE_DIALOG);
-  gtk_table_attach (GTK_TABLE (table), image, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
+  gtk_grid_attach (GTK_GRID (grid), image, 0, 0, 1, 1);
   gtk_widget_show (image);
 
   /* add the header label */
   label = gtk_label_new (_("Please select your preferred archive manager\nfrom the list of available applications below:"));
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0f, 0.5f);
-  gtk_table_attach (GTK_TABLE (table), label, 1, 2, 0, 1, GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
+  gtk_widget_set_vexpand (label, TRUE);
+  gtk_grid_attach (GTK_GRID (grid), label, 1, 0, 1, 1);
   gtk_widget_show (label);
 
   /* add the button box */
-  bbox = gtk_vbox_new (FALSE, 6);
+  bbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
   gtk_container_set_border_width (GTK_CONTAINER (bbox), 12);
-  gtk_table_attach (GTK_TABLE (table), bbox, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+  gtk_widget_set_hexpand (bbox, TRUE);
+  gtk_widget_set_vexpand (bbox, TRUE);
+  gtk_grid_attach (GTK_GRID (grid), bbox, 1, 1, 1, 1);
   gtk_widget_show (bbox);
 
   /* add the radio buttons */
@@ -128,11 +125,11 @@ tap_backend_mime_ask (GList     *mime_applications,
       space = strchr (command, ' ');
       if (G_LIKELY (space != NULL))
         *space = '\0';
-      gtk_tooltips_set_tip (tooltips, button, command, NULL);
+      gtk_widget_set_tooltip_text (button, command);
       g_free (command);
 
       /* add the hbox */
-      hbox = gtk_hbox_new (FALSE, 2);
+      hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
       gtk_container_add (GTK_CONTAINER (button), hbox);
       gtk_widget_show (hbox);
 
@@ -148,7 +145,7 @@ tap_backend_mime_ask (GList     *mime_applications,
 
       /* add the label for the application */
       label = gtk_label_new (g_app_info_get_name (mp->data));
-      gtk_misc_set_alignment (GTK_MISC (label), 0.0f, 0.5f);
+      gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
       gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
       gtk_widget_show (label);
     }
@@ -167,7 +164,6 @@ tap_backend_mime_ask (GList     *mime_applications,
     }
 
   /* cleanup */
-  g_object_unref (G_OBJECT (tooltips));
   gtk_widget_destroy (dialog);
 
   return mime_application;
@@ -402,6 +398,7 @@ tap_backend_run (const gchar *action,
   GList                    *lp;
   GPid                      pid = -1;
   gint                      n;
+  gchar                    *display = NULL;
 
   /* determine the mime infos on-demand */
   if (G_LIKELY (content_types == NULL))
@@ -445,12 +442,16 @@ tap_backend_run (const gchar *action,
           /* determine the screen for this window */
           screen = gtk_widget_get_screen (window);
 
+          if (screen != NULL)
+            display = gdk_screen_make_display_name (screen);
+
           /* try to run the command */
-          if (!gdk_spawn_on_screen (screen, folder, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid, error))
+          if (!g_spawn_async (folder, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, tap_backend_setup_display_cb, display, &pid, error))
             pid = -1;
 
           /* cleanup */
           g_strfreev (argv);
+          g_free (display);
         }
 
       /* cleanup */
@@ -466,6 +467,14 @@ tap_backend_run (const gchar *action,
 
 
 
+static void
+tap_backend_setup_display_cb (gpointer data)
+{
+  g_setenv ("DISPLAY", (char *) data, TRUE);
+}
+
+
+
 /**
  * tap_backend_create_archive:
  * @folder : the path to the folder in which to create the archive.
@@ -577,5 +586,3 @@ tap_backend_extract_to (const gchar *folder,
   /* run the action */
   return tap_backend_run ("extract-to", folder, files, NULL, window, error);
 }
-
-
